#   save.r    Saving Lives and Years of Life by COVID-19 Vaccination
#       21-24  May 2021;   Kenneth W. Wachter
#       Sample R-code for the Formal Demography Workshop 2021
#                 University of California, Berkeley
#                 Wednesday, 26 May 2021,  9:00 to 10:30 A.M. 
#


#####################################################################
#      Section 1:    Shape of the Logistic Log Hazard
#             Compute the Logistic Hazard and Cumulative Hazard
#             as functions of age,  and observe the downward-bending
#             shape of the log hazard as a function of age. 
#             This special case is stylized to make formulas simple.
#             For greater realism, picture one unit in x standing
#             for ten years of age, starting around age 60,
#             for a somewhat frail subset of the population.
#####################################################################

dx   <-  0.01                   #  step size for age variable 
x    <-  seq(0,8, by = dx)      #  age variable 

h    <-  1/(1 + exp(-x))        #  hazard

H    <-  log( (1 + exp(x))/2)   #  cumulative hazard  

ell  <-  exp(-H)                #  survivorship  

#                Check the shape of the hazard and cumulative hazard .
plot( x,  h,      type = "l",  xlim=c(-4,4), ylim = c(0,1), lwd=2 )
title(  main = "Hazard as Function of Age" ) 
abline( v = 0)                # vertical line at starting age
lines( -x, 1-h, lty = 2 )     # dashed line for hazard before start 


plot( x,  H,      type = "l", xlim = c(-0,4), ylim = c(0,4)   )
title(  main = "Cumulative Hazard as Function of Age" ) 

#                Now observe the downward bending of the log hazard
plot( x,  log(h), type = "l",  xlim = c(0,4)    )
title(  main = "Log Hazard as Function of Age" ) 

#        For the logistic, the slope of the log hazard
#           is visibly decreasing over the whole range,
#           so the second derivative of the log hazard
#           is always negative,  and the sufficient
#           condition for monotone increasing yearsaving
#           is satisfied.



#####################################################################
#    Section 2:  The Hazard as a Function of the Cumulative Hazard
#         Observe the change in horizontal separations
#####################################################################


#                Display the hazard as a function of the cumulative hazard
plot( H,  h,      type = "l",  xlim=c(0, 4), ylim = c(0,1), lwd=2 )
title(  main = "Hazard Little-h as a Function of Big-H" ) 


# Select a few points for a closer look at separations
J   <-  c(1, 51,101, 151)      # indices for selected points 

#               Selected hazard values as a function of age x.
plot( x[J], h[J], xlim=c(0,1.5),  ylim = c(0.4,1.1 ), type="n" ) 
title( main = "Hazard Values in X-space" ) 
points( x[J], h[J] )
arrows( x[J], 0*J, x[J], h[J] )


#                   The same hazard values as a function of H. 
plot( H[J], h[J], xlim=c(0,1.5),  ylim = c(0.4,1.1 ), type="n" ) 
title( main = "The Same Hazard Values in H-space" ) 
points( H[J], h[J] )
arrows( H[J], 0*J, H[J], h[J] )
#                   Compare the two plots 

###>>>       How do these plots differ?

###>>>       Measure the differences,  using the R-function diff  or otherwise.


#####################################################################
#         Section 3:  Introducing the ``Big-H Assist''  
#            We want values of h at a set of values of H that are
#            equally spaced,  so that we can compute areas as sums.
#            Use the approx function in R to go from a chosen value
#            of  H  to the corresponding value of h.  We call it lambda.
#####################################################################


#             Illustrate the process of finding h from H, 
#                   that is, finding lambda(u)  for H = u   
u     <-  0.5               #  trial starting value in H-space

plot( H, h,  xlim = c(-0.3, 2)  ) #
title(  main = "Finding lambda(u) when u = 0.5" ) 

lines(  c(u, u, -0.3),  c( 0, 0.696733, 0.696733), lty = 3)  
arrows ( 0, 0.696733, -0.3,  0.696733  )  

lambda_u     <-  approx( H, h, u)$y  


#####################################################################
#   Section 4.    Evaluate lambda over a grid of values of t
#          For a trial value of u,  plot the curve for the integrand. 
#          The area under the curve is the value of yearsaving
#          for the age corresponding to this value of u.#
#####################################################################

dt   <-  0.01               #  step size in H-units

ttt  <- seq(0,4, by = dt)   #  equally spaced points in H-space

u            <-  0.5  

lambda_u     <-  approx( H, h, u)$y  

lambda_ut    <-  approx( H, h, u+ttt)$y 

ratio        <-  lambda_u/lambda_ut


plot( ttt,  exp(-ttt)*ratio, type = "l", lty =2, bty = "n"  )
lines( c(0,0,4), c(1,0,0),   lty = 2 )
title( main = "Integrand When  u = 1/2"  )

#               Rough value of yearsaving for this starting point

grough   <-   sum( exp(-ttt)*ratio*dt )   

###>>>>      Plot   lambda_ut versus ttt
###          and compare   1 - (1/2)\exp(-u)\exp(-ttt) versus ttt


#####################################################################
#      Section 5:   Plot the ratio lambda(u)/lambda(u+t)
#           as a function of t for several values of u.
#           Observe that a higher value of u determines 
#           a ratio curve that is higher  AT EVERY VALUE OF t.
#####################################################################

plot( ttt,  ratio, type = "l", ylim = c(0,1), bty = "n"  )
title( main = "Ratio Curve as a Function of t for selected values of u")


u    <- .1                 # New starting vaue u in H-space 

lambda_u     <-  approx( H, h, u)$y  

lambda_ut    <-  approx( H, h, u+ttt)$y 

lines( ttt, lambda_u/lambda_ut, lty = 2 ) 

#>>>>            Add some more lines for values of u.

#            Observe that increasing u drives the ratio up 
#                FOR ALL VALUES OF t  
#                and therefore higher u means higher yearsaving.
#            Yearsaving is a monotone increasing function of u
#                and therefore a monotone increasing function of age
#                in this case.


